"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint max-depth: ["error", 5] */
const command_1 = require("@oclif/command");
const fs = require("fs");
const path = require("path");
const date_fns_1 = require("date-fns");
const octokit_1 = require("octokit");
const base_1 = require("../base");
const searchIssues_1 = require("../utils/jira/searchIssues");
const getStreamsCachePerTeam_1 = require("../utils/streams/getStreamsCachePerTeam");
const createDistributionTable_1 = require("../utils/report/createDistributionTable");
const createReleasePipelineTable_1 = require("../utils/report/createReleasePipelineTable");
const getInitiatives_1 = require("../utils/report/getInitiatives");
const createInitiativesOverviewTable_1 = require("../utils/report/createInitiativesOverviewTable");
const createInitiativeDetails_1 = require("../utils/report/createInitiativeDetails");
class Report extends base_1.default {
    async run() {
        const { flags } = this.parse(Report);
        const userConfig = this.userConfig;
        const cacheDir = path.join(this.config.configDir, 'cache');
        // Get data from Jira
        this.log('Getting Release pipeline issues from Jira');
        const releaseTicketsRaw = await (0, searchIssues_1.default)(userConfig.jira, userConfig.report.releasePipeline.jql, '*all,-comments,labels,created,summary,status,issuetype,assignee,project,resolution,resolutiondate,fixVersion');
        // For every single release, get the issues part of the release
        const releaseTickets = [];
        for (const release of releaseTicketsRaw) {
            const issues = [];
            if (release.fields.fixVersions !== undefined &&
                release.fields.fixVersions.length > 0) {
                const issuesJira = await (0, searchIssues_1.default)(userConfig.jira, `fixVersion = "${release.fields.fixVersions[0].name}"`, '*all,-comments,labels,created,summary,status,issuetype,assignee,project,resolution,resolutiondate');
                for (const issue of issuesJira) {
                    if (issue.key !== release.key) {
                        issues.push({
                            key: issue.key,
                            status: issue.fields.status,
                        });
                    }
                }
            }
            releaseTickets.push(Object.assign(Object.assign({}, release), { issuesInRelease: issues }));
        }
        this.log('Getting initiatives from Jira');
        const initiativesSourceTickets = await (0, searchIssues_1.default)(userConfig.jira, userConfig.report.initiatives.jql, `*all,comments,labels,created,summary,status,issuetype,assignee,project,resolution,resolutiondate,${userConfig.jira.fields.deliveryConfidence},${userConfig.jira.fields.expectedDelivery}`);
        const teams = (0, getStreamsCachePerTeam_1.getStreamsCachePerTeam)(userConfig, cacheDir);
        const initiatives = (0, getInitiatives_1.getInitiatives)(teams, initiativesSourceTickets, userConfig);
        // Begin by generating a report object
        // This report will then be used to create the markdown
        const deliveryReportSrc = {
            date: new Date(),
            teams: teams.filter(t => userConfig.report.distributionTeams.includes(t.name)),
            releaseTickets,
            initiatives,
        };
        const reportName = `Delivery report for week: ${(0, date_fns_1.format)(deliveryReportSrc.date, 'yyyy-ww')}`;
        let deliveryReport = `# ${reportName} \n`;
        deliveryReport += `<sub>🕟 Report generated on: ${(0, date_fns_1.format)(deliveryReportSrc.date, 'LLLL dd, yyyy HH:mm')}. Initiatives listed in this report are [fetched from Jira](${encodeURI(`${userConfig.jira.host}/issues/?jql=${userConfig.report.initiatives.jql}`)}). </sub> \n`;
        deliveryReport += `## Overview \n`;
        deliveryReport += `### Effort distribution \n`;
        deliveryReport += (0, createDistributionTable_1.createDistributionTable)(deliveryReportSrc.teams, userConfig);
        deliveryReport += `\n`;
        deliveryReport += `### Releases pipeline \n`;
        deliveryReport += (0, createReleasePipelineTable_1.createReleasePipelineTable)(deliveryReportSrc.releaseTickets, userConfig);
        deliveryReport += `\n`;
        deliveryReport += `### Initiatives overview \n`;
        deliveryReport += (0, createInitiativesOverviewTable_1.createInitiativesOverviewTable)(deliveryReportSrc.initiatives, userConfig);
        deliveryReport += `# Initiatives Details \n`;
        for (const i of deliveryReportSrc.initiatives) {
            deliveryReport += (0, createInitiativeDetails_1.createInitiativeDetails)(i, userConfig);
            deliveryReport += `---\n`;
        }
        deliveryReport += `\n`;
        deliveryReport += `<sub>ℹ️ You can hide comments from the report by adding #NO-REPORT anywhere in a comment.</sub> \n`;
        deliveryReportSrc.markdown = deliveryReport;
        if (flags.save) {
            // Saving the report to file
            const reportsCache = path.join(cacheDir, `delivery-reports/`);
            fs.mkdirSync(reportsCache, { recursive: true });
            const cacheFilename = `${(0, date_fns_1.format)(new Date(), 'yyyy-ww')}.json`;
            this.log(`Saving report to cache file: ${path.join(reportsCache, cacheFilename)}`);
            const fileStream = fs.createWriteStream(path.join(reportsCache, cacheFilename), { flags: 'w' });
            fileStream.write(JSON.stringify(deliveryReportSrc));
            fileStream.end();
            // Create/Update report issue in GitHub
            const octokit = new octokit_1.Octokit({ auth: userConfig.report.github.token });
            const issues = await octokit.paginate(`GET /repos/{owner}/{repo}/issues?labels=${userConfig.report.github.label}`, {
                owner: userConfig.report.github.owner,
                repo: userConfig.report.github.repository,
                state: 'all',
                per_page: 100,
            });
            // Search for an issue with the same title
            // If present it will be updated, if absent it will be created.
            const existingReportIssue = issues.find((i) => i.title === reportName);
            if (existingReportIssue === undefined || existingReportIssue === null) {
                // Create Issue
                await octokit.request('POST /repos/{owner}/{repo}/issues', {
                    owner: userConfig.report.github.owner,
                    repo: userConfig.report.github.repository,
                    title: reportName,
                    body: deliveryReport,
                    labels: [userConfig.report.github.label],
                });
            }
            else if (existingReportIssue.state === 'open') {
                // Update Issue
                await octokit.request('PATCH /repos/{owner}/{repo}/issues/{issue_number}', {
                    owner: userConfig.report.github.owner,
                    repo: userConfig.report.github.repository,
                    issue_number: existingReportIssue.number,
                    title: reportName,
                    body: deliveryReport,
                });
                this.log(`Updated GitHub Issue # ${existingReportIssue.number} (${existingReportIssue.html_url})`);
            }
            else {
                this.log(`GitHub Issue # ${existingReportIssue.number} is closed, not updating it (${existingReportIssue.html_url}).`);
            }
        }
    }
}
exports.default = Report;
Report.description = 'Fetches Completion and Forecast streams per team';
Report.flags = Object.assign(Object.assign({}, base_1.default.flags), { help: command_1.flags.help({ char: 'h' }), save: command_1.flags.boolean({
        char: 's',
        default: false,
        description: 'Save a cache file and submit data to GitHub Issues',
    }) });
